import java.io.FileReader;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.FileNotFoundException;
import java.util.Scanner;

/*
  Usage:
  * Call the static read method to prompt the user for the file containing the
    positions.
  * The method will prompt for a filename and try to open that the file in the
    location where the Driver for your Java program is located.
  * The prompt will be repeated if there are problems accessing the file (e.g.
    the file is not at the specified location, it is corrupted etc.).
  * The method returns a reference to a 2D array of references to objects of type
    Entity.
  * The appearance attribute of the entity will correspond to the appearance of
    characters in the file.
  * Starting files must be exactly 10x10 characters in size and consist only of the
    'O' character, an 'E' character or a space.

  Version Feb 16B, 2021
  * Program repeats prompt for filename if file not found at location.

  Version Feb 16A, 2021
  * Program allows the user to specify the input file at runtime.

  Version: Feb 11A:
  * Changed references of dwarves to elves. Constants related to the
  * Entity attributes deleted.
*/

// this file takes in the txt file and initializes the world which is called in the world class
// this is where we get the values
public class FileInitialization
{
    public static Entity[][] read() {
        Entity [][] temp = new Entity[World.SIZE][World.SIZE];
        String line; // changed to this as it's going to be null regardless
        int row, column; // changed from r and c so its more readable
        char letter;
        FileContainer aContainer; // changed to this as it's going to be null regardless
        try {
            aContainer = checkingFile();
            BufferedReader bufferReader = aContainer.getBufferedReader(); //changed name from br
            FileReader fileReader = aContainer.getFileReader(); // changed name from br so its more readable
            line = bufferReader.readLine();
            if (line == null)
                System.out.println("Empty file, nothing to read");
            row = 0;
            while (line != null) {
                column = 0;
                while(column < World.SIZE) {
                    letter = line.charAt(column);
                    temp[row][column] = createEntity(letter);
                    column = column + 1;
                }
                line = bufferReader.readLine();
                if (line != null)
                    row = row + 1;
            }
            fileReader.close();
        }
        catch (FileNotFoundException e) {
            String location = System.getProperty("user.dir");
            System.out.println("Input file not in " + location);
        }
        catch (IOException e) {
            System.out.println("General file input problem " +
                    "reading starting positions");
        }
        return(temp);
    }

    private static FileContainer checkingFile()
    {
        FileContainer aContainer; // changed to this as it's going to be null regardless
        Scanner in = new Scanner(System.in);
        String filename; // changed to this as it's going to be null regardless
        while (true) // changed to this as it's still if false
        {
            try
            {
                System.out.print("Name of file containing starting positions: ");
                filename = in.nextLine();
                FileReader fileReader = new FileReader(filename);
                BufferedReader bufferReader = new BufferedReader(fileReader);
                aContainer = new FileContainer(bufferReader, fileReader);
                return aContainer;
            }
            catch (FileNotFoundException ex)
            {
                String location = System.getProperty("user.dir");
                System.out.println("Input file not in " + location);
            }

        }
    }

    private static Entity createEntity(char letter) {
        return switch (letter) {
            case 'O' -> new Orc();
            case 'E' -> new Elf();
            case Entity.EMPTY -> null;
            default -> {
                System.out.println("Error: Invalid character [" + letter + "] in file");
                yield null;  // 'yield' is used to return a value from the block
            }
        };
    }
}