import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import java.io.ByteArrayOutputStream;
import java.io.PrintStream;
import static org.junit.jupiter.api.Assertions.*;

public class OrderProcessorTest {

    private OrderProcessor orderProcessor;

    @BeforeEach
    public void setUp() {
        orderProcessor = new OrderProcessor();
    }

    @Test
    public void testProcessOrderWithValidStock() {
        // Setup
        String customerName = "Alice";
        String item = "Laptop";
        int quantity = 2;
        double price = 750.00;

        // Use a ByteArrayOutputStream to capture System.out
        ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
        System.setOut(new PrintStream(outputStream));

        // Call the method
        orderProcessor.processOrder(customerName, item, quantity, price);

        // Check the output
        String expectedOutput = "Processing order for: Alice\r\n" +
                "Total price: $1500.0\r\n" +
                "Processing payment...\r\n" +
                "High-value transaction! Notifying manager...\r\n" +
                "Payment successful.\r\n" +
                "Preparing Laptop for shipping...\r\n" +
                "Order for Alice has been shipped.\r\n";
        assertEquals(expectedOutput, outputStream.toString());
    }

    @Test
    public void testProcessOrderWithInsufficientStock() {
        // Setup
        String customerName = "Bob";
        String item = "Laptop";
        int quantity = 15;  // Greater than 10, so stock will be insufficient
        double price = 750.00;

        // Use a ByteArrayOutputStream to capture System.out
        ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
        System.setOut(new PrintStream(outputStream));

        // Call the method
        orderProcessor.processOrder(customerName, item, quantity, price);

        // Check the output
        String expectedOutput = "Processing order for: Bob\r\n" +
                "Total price: $11250.0\r\n" +
                "Insufficient stock for Laptop\r\n";
        assertEquals(expectedOutput, outputStream.toString());
    }
}
